classdef WOK_Aufgabe4p3_WT21_exported < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                     matlab.ui.Figure
        GridLayout                   matlab.ui.container.GridLayout
        LeftPanel                    matlab.ui.container.Panel
        s1Knob                       matlab.ui.control.Knob
        s2Knob                       matlab.ui.control.Knob
        Gsss4s3s4Label               matlab.ui.control.Label
        RsKss2ss1Label               matlab.ui.control.Label
        KSlider                      matlab.ui.control.Slider
        Aufgabe43Label               matlab.ui.control.Label
        KEditFieldLabel              matlab.ui.control.Label
        KEditField                   matlab.ui.control.NumericEditField
        s1EditFieldLabel             matlab.ui.control.Label
        s1EditField                  matlab.ui.control.NumericEditField
        s2EditFieldLabel             matlab.ui.control.Label
        s2EditField                  matlab.ui.control.NumericEditField
        Image_2                      matlab.ui.control.Image
        RightPanel                   matlab.ui.container.Panel
        UIAxes                       matlab.ui.control.UIAxes
        REGELUNGSTECHNIKWT2021Label  matlab.ui.control.Label
        UniversitaetderBundeswehrMuenchenLRT15Label  matlab.ui.control.Label
        VictorCheiddeChaimLabel      matlab.ui.control.Label
        Image                        matlab.ui.control.Image
    end

    % Properties that correspond to apps with auto-reflow
    properties (Access = private)
        onePanelWidth = 576;
    end

    
    properties (Access = private)
        s = tf([1 0],1);
        G = tf([1 0],[1 3 -16 -48]) % Pole: -4, -3, 4; Nullstelle: 0.
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function startupFcn(app)
           
            [R,K] = rlocus(app.G);
            gainToMark = app.KSlider.Value;
            
            hold(app.UIAxes,'on')
            
            plot(app.UIAxes,real(R(1,:)),imag(R(1,:)),'LineWidth', 3);
            plot(app.UIAxes,real(R(2,:)),imag(R(2,:)),'LineWidth', 3);
            plot(app.UIAxes,real(R(3,:)),imag(R(3,:)),'LineWidth', 3);
            
            plot(app.UIAxes,-4,0,'bx','MarkerSize',14);
            plot(app.UIAxes,-3,0,'bx','MarkerSize',14);
            plot(app.UIAxes,4,0,'bx','MarkerSize',14);
            plot(app.UIAxes,0,0,'bo','MarkerSize',14);
            
            indexOfGain = find(K >= gainToMark,1,'first');
           
            plot(app.UIAxes,real(R(1,indexOfGain)),imag(R(1,indexOfGain)),'ms','MarkerSize',10);
            plot(app.UIAxes,real(R(2,indexOfGain)),imag(R(2,indexOfGain)),'ms','MarkerSize',10);
            plot(app.UIAxes,real(R(3,indexOfGain)),imag(R(3,indexOfGain)),'ms','MarkerSize',10);
            
            hold(app.UIAxes,'off')
            
        end

        % Changes arrangement of the app based on UIFigure width
        function updateAppLayout(app, event)
            currentFigureWidth = app.UIFigure.Position(3);
            if(currentFigureWidth <= app.onePanelWidth)
                % Change to a 2x1 grid
                app.GridLayout.RowHeight = {467, 467};
                app.GridLayout.ColumnWidth = {'1x'};
                app.RightPanel.Layout.Row = 2;
                app.RightPanel.Layout.Column = 1;
            else
                % Change to a 1x2 grid
                app.GridLayout.RowHeight = {'1x'};
                app.GridLayout.ColumnWidth = {253, '1x'};
                app.RightPanel.Layout.Row = 1;
                app.RightPanel.Layout.Column = 2;
            end
        end

        % Value changed function: KSlider, s1Knob, s2Knob
        function KnobValueChanged(app, event)
            
            cla(app.UIAxes);
            
            s1 = app.s1Knob.Value;
            s2 = app.s2Knob.Value;
            app.s1EditField.Value = s1;
            app.s2EditField.Value = s2;
            
            Rs = (app.s - s2)/(app.s - s1);
            
            [R,K] = rlocus(app.G*Rs);
            gainToMark = app.KSlider.Value;
            app.KEditField.Value = gainToMark;
            indexOfGain = find(K >= gainToMark,1,'first');
            [m,~] = size(R);
            
            hold(app.UIAxes,'on')
            
            plot(app.UIAxes,real(R(1,:)),imag(R(1,:)),'LineWidth', 3);
            plot(app.UIAxes,real(R(2,:)),imag(R(2,:)),'LineWidth', 3);
            plot(app.UIAxes,real(R(3,:)),imag(R(3,:)),'LineWidth', 3);
            
            if m > 3
            plot(app.UIAxes,real(R(4,:)),imag(R(4,:)),'LineWidth', 3);
            plot(app.UIAxes,real(R(4,indexOfGain)),imag(R(4,indexOfGain)),'ms','MarkerSize',10);
            end
            
            plot(app.UIAxes,-4,0,'bx','MarkerSize',14);
            plot(app.UIAxes,-3,0,'bx','MarkerSize',14);
            plot(app.UIAxes,4,0,'bx','MarkerSize',14);
            plot(app.UIAxes,s1,0,'gx','MarkerSize',14);
            
            plot(app.UIAxes,0,0,'bo','MarkerSize',14);
            plot(app.UIAxes,s2,0,'go','MarkerSize',14);
            
            plot(app.UIAxes,real(R(1,indexOfGain)),imag(R(1,indexOfGain)),'ms','MarkerSize',10);
            plot(app.UIAxes,real(R(2,indexOfGain)),imag(R(2,indexOfGain)),'ms','MarkerSize',10);
            plot(app.UIAxes,real(R(3,indexOfGain)),imag(R(3,indexOfGain)),'ms','MarkerSize',10);
            
            hold(app.UIAxes,'off')
                        
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.AutoResizeChildren = 'off';
            app.UIFigure.Position = [100 100 787 467];
            app.UIFigure.Name = 'UI Figure';
            app.UIFigure.SizeChangedFcn = createCallbackFcn(app, @updateAppLayout, true);

            % Create GridLayout
            app.GridLayout = uigridlayout(app.UIFigure);
            app.GridLayout.ColumnWidth = {253, '1x'};
            app.GridLayout.RowHeight = {'1x'};
            app.GridLayout.ColumnSpacing = 0;
            app.GridLayout.RowSpacing = 0;
            app.GridLayout.Padding = [0 0 0 0];
            app.GridLayout.Scrollable = 'on';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.GridLayout);
            app.LeftPanel.BackgroundColor = [1 1 1];
            app.LeftPanel.Layout.Row = 1;
            app.LeftPanel.Layout.Column = 1;

            % Create s1Knob
            app.s1Knob = uiknob(app.LeftPanel, 'continuous');
            app.s1Knob.Limits = [-5 5];
            app.s1Knob.ValueChangedFcn = createCallbackFcn(app, @KnobValueChanged, true);
            app.s1Knob.Position = [37 85 60 60];

            % Create s2Knob
            app.s2Knob = uiknob(app.LeftPanel, 'continuous');
            app.s2Knob.Limits = [-5 5];
            app.s2Knob.ValueChangedFcn = createCallbackFcn(app, @KnobValueChanged, true);
            app.s2Knob.Position = [154 85 60 60];

            % Create Gsss4s3s4Label
            app.Gsss4s3s4Label = uilabel(app.LeftPanel);
            app.Gsss4s3s4Label.BackgroundColor = [1 1 1];
            app.Gsss4s3s4Label.HorizontalAlignment = 'center';
            app.Gsss4s3s4Label.FontSize = 16;
            app.Gsss4s3s4Label.FontWeight = 'bold';
            app.Gsss4s3s4Label.FontColor = [0 0 1];
            app.Gsss4s3s4Label.Position = [31 316 200 30];
            app.Gsss4s3s4Label.Text = 'G(s) = s/((s+4)(s+3)(s-4))';

            % Create RsKss2ss1Label
            app.RsKss2ss1Label = uilabel(app.LeftPanel);
            app.RsKss2ss1Label.BackgroundColor = [1 1 1];
            app.RsKss2ss1Label.HorizontalAlignment = 'center';
            app.RsKss2ss1Label.FontSize = 16;
            app.RsKss2ss1Label.FontWeight = 'bold';
            app.RsKss2ss1Label.FontColor = [0.3922 0.8314 0.0745];
            app.RsKss2ss1Label.Position = [31 286 200 30];
            app.RsKss2ss1Label.Text = 'R(s) = K*(s-s2)/(s-s1)';

            % Create KSlider
            app.KSlider = uislider(app.LeftPanel);
            app.KSlider.Limits = [0 50];
            app.KSlider.ValueChangedFcn = createCallbackFcn(app, @KnobValueChanged, true);
            app.KSlider.Position = [21 233 210 3];
            app.KSlider.Value = 1;

            % Create Aufgabe43Label
            app.Aufgabe43Label = uilabel(app.LeftPanel);
            app.Aufgabe43Label.BackgroundColor = [0.9412 0.9412 0.9412];
            app.Aufgabe43Label.FontSize = 18;
            app.Aufgabe43Label.FontWeight = 'bold';
            app.Aufgabe43Label.FontColor = [0.149 0.149 0.149];
            app.Aufgabe43Label.Position = [12 356 110 26];
            app.Aufgabe43Label.Text = 'Aufgabe 4.3:';

            % Create KEditFieldLabel
            app.KEditFieldLabel = uilabel(app.LeftPanel);
            app.KEditFieldLabel.HorizontalAlignment = 'right';
            app.KEditFieldLabel.FontSize = 16;
            app.KEditFieldLabel.FontWeight = 'bold';
            app.KEditFieldLabel.Position = [81 254 31 22];
            app.KEditFieldLabel.Text = 'K =';

            % Create KEditField
            app.KEditField = uieditfield(app.LeftPanel, 'numeric');
            app.KEditField.Limits = [0 Inf];
            app.KEditField.ValueDisplayFormat = '%.2f';
            app.KEditField.Editable = 'off';
            app.KEditField.FontSize = 16;
            app.KEditField.Position = [121 254 50 22];
            app.KEditField.Value = 1;

            % Create s1EditFieldLabel
            app.s1EditFieldLabel = uilabel(app.LeftPanel);
            app.s1EditFieldLabel.HorizontalAlignment = 'right';
            app.s1EditFieldLabel.FontSize = 16;
            app.s1EditFieldLabel.FontWeight = 'bold';
            app.s1EditFieldLabel.Position = [12 36 37 22];
            app.s1EditFieldLabel.Text = 's1 =';

            % Create s1EditField
            app.s1EditField = uieditfield(app.LeftPanel, 'numeric');
            app.s1EditField.Limits = [-5 5];
            app.s1EditField.ValueDisplayFormat = '%.2f';
            app.s1EditField.Editable = 'off';
            app.s1EditField.FontSize = 16;
            app.s1EditField.Position = [58 36 50 22];

            % Create s2EditFieldLabel
            app.s2EditFieldLabel = uilabel(app.LeftPanel);
            app.s2EditFieldLabel.HorizontalAlignment = 'right';
            app.s2EditFieldLabel.FontSize = 16;
            app.s2EditFieldLabel.FontWeight = 'bold';
            app.s2EditFieldLabel.Position = [133 36 37 22];
            app.s2EditFieldLabel.Text = 's2 =';

            % Create s2EditField
            app.s2EditField = uieditfield(app.LeftPanel, 'numeric');
            app.s2EditField.Limits = [-5 5];
            app.s2EditField.ValueDisplayFormat = '%.2f';
            app.s2EditField.Editable = 'off';
            app.s2EditField.FontSize = 16;
            app.s2EditField.Position = [179 36 50 22];

            % Create Image_2
            app.Image_2 = uiimage(app.LeftPanel);
            app.Image_2.Position = [11 410 220 46];
            app.Image_2.ImageSource = 'UniBwlogo.png';

            % Create RightPanel
            app.RightPanel = uipanel(app.GridLayout);
            app.RightPanel.BackgroundColor = [1 1 1];
            app.RightPanel.Layout.Row = 1;
            app.RightPanel.Layout.Column = 2;

            % Create UIAxes
            app.UIAxes = uiaxes(app.RightPanel);
            title(app.UIAxes, 'Wurzelortskurve - WOK')
            xlabel(app.UIAxes, 'Re')
            ylabel(app.UIAxes, 'Im')
            app.UIAxes.XLim = [-6 6];
            app.UIAxes.YLim = [-20 20];
            app.UIAxes.XAxisLocation = 'origin';
            app.UIAxes.XTick = [-6 -5 -4 -3 -2 -1 0 1 2 3 4 5 6];
            app.UIAxes.XTickLabel = {''; '-5'; '-4'; '-3'; '-2'; '-1'; ''; '1'; '2'; '3'; '4'; '5'; ''};
            app.UIAxes.XMinorTick = 'on';
            app.UIAxes.YAxisLocation = 'origin';
            app.UIAxes.YTick = [-20 -15 -10 -5 0 5 10 15 20];
            app.UIAxes.YTickLabel = {'-20'; '-15'; '-10'; '-5'; ''; '5'; '10'; '15'; '20'};
            app.UIAxes.YMinorTick = 'on';
            app.UIAxes.Position = [21 69 490 330];

            % Create REGELUNGSTECHNIKWT2021Label
            app.REGELUNGSTECHNIKWT2021Label = uilabel(app.RightPanel);
            app.REGELUNGSTECHNIKWT2021Label.FontSize = 18;
            app.REGELUNGSTECHNIKWT2021Label.FontWeight = 'bold';
            app.REGELUNGSTECHNIKWT2021Label.Position = [21 424 287 22];
            app.REGELUNGSTECHNIKWT2021Label.Text = 'REGELUNGSTECHNIK - WT2021';

            % Create UniversitaetderBundeswehrMuenchenLRT15Label
            app.UniversitaetderBundeswehrMuenchenLRT15Label = uilabel(app.RightPanel);
            app.UniversitaetderBundeswehrMuenchenLRT15Label.FontWeight = 'bold';
            app.UniversitaetderBundeswehrMuenchenLRT15Label.Position = [21 24 285 22];
            app.UniversitaetderBundeswehrMuenchenLRT15Label.Text = 'Universitaet der Bundeswehr Muenchen - LRT15';

            % Create VictorCheiddeChaimLabel
            app.VictorCheiddeChaimLabel = uilabel(app.RightPanel);
            app.VictorCheiddeChaimLabel.FontWeight = 'bold';
            app.VictorCheiddeChaimLabel.Position = [21 4 130 22];
            app.VictorCheiddeChaimLabel.Text = 'Victor Cheidde Chaim';

            % Create Image
            app.Image = uiimage(app.RightPanel);
            app.Image.Position = [421 12 90 46];
            app.Image.ImageSource = 'LRT15_logo.png';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = WOK_Aufgabe4p3_WT21_exported

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            % Execute the startup function
            runStartupFcn(app, @startupFcn)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end